package com.winhc.bigdata.spark.ng.change

import com.winhc.bigdata.spark.utils.BaseUtil.cleanup
import org.apache.spark.internal.Logging

import scala.annotation.meta.{getter, setter}

/**
 * @Author: XuJiakai
 * @Date: 2020/7/9 16:44
 * @Description:
 */

trait NgCompanyChangeHandle extends Serializable with Logging {
  @getter
  @setter
  protected val equCols: Seq[String]

  /**
   * 主入口函数
   *
   * @param rowkey
   * @param oldMap
   * @param newMap
   * @return rowkey,company_id,类型【insert or update or deleted】,老数据,新数据，更新字段，更新标题，数据标签,业务时间
   */
  def handle(rowkey: String, oldMap: Map[String, String], newMap: Map[String, String]): (String, String, String, Map[String, String], Map[String, String], String, String, String, String) = {
    var update_type: NgCompanyUpdateType.UpdateType = null

    val company_id = getCompanyId(rowkey, newMap)
    var update_fields: String = null

    var title: String = null
    val label = getLabel(oldMap, newMap)
    val biz_time = getBizDate(newMap)

    if (oldMap == null) {
      newMap.getOrElse("deleted", "0") match {
        case "0" => {
          update_type = NgCompanyUpdateType.Insert
          title = getInsertTitle(newMap)
        }
        case _ => null
      }
    } else {
      val new_deleted = newMap.getOrElse("deleted", "0")
      val old_deleted = oldMap.getOrElse("deleted", "0")

      s"$old_deleted$new_deleted" match {
        case "00" => {
          val t = getEquAndFields(oldMap, newMap)
          if (!t._1) {
            update_type = NgCompanyUpdateType.Update
            title = getUpdateTitle(newMap)
            update_fields = t._2
          }
        }
        case "01" => {
          update_type = NgCompanyUpdateType.Deleted
          title = getDeletedTitle(newMap)
        }
        case "10" | "90" | "09" | "19" => null

        case _ => null
      }
    }

    if (update_type == null || title == null || biz_time == null) {
      return null
    }
    (rowkey, company_id, update_type.toString, oldMap, newMap,
      update_fields
      , title, label, biz_time)
  }

  /**
   * 获取公司company_id，默认为rowkey前半段
   *
   * @param rowkey
   * @param newMap
   * @return
   */
  protected def getCompanyId(rowkey: String, newMap: Map[String, String]): String = rowkey.split("_")(0)

  /**
   * 如果数据发生更新，则显示该标题
   *
   * @param newMap
   * @return
   */
  protected def getUpdateTitle(newMap: Map[String, String]): String

  /**
   * 如果数据发生新增，则显示该标题
   *
   * @param newMap
   * @return
   */
  protected def getInsertTitle(newMap: Map[String, String]): String

  /**
   * 如果数据发生移除，则显示该标题，deleted:x->9 删除不算
   *
   * @param newMap
   * @return
   */
  protected def getDeletedTitle(newMap: Map[String, String]): String

  /**
   * 用于展示的标签
   *
   * @param oldMap
   * @param newMap
   * @return
   */
  protected def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String

  /**
   * 获取变更的业务时间
   *
   * @param newMap
   * @return
   */
  protected def getBizDate(newMap: Map[String, String]): String

  /**
   * 判断哪些字段不相同
   *
   * @param oldMap
   * @param newMap
   * @return true（相同） 差异字段
   */
  protected def getEquAndFields(oldMap: Map[String, String], newMap: Map[String, String]): (Boolean, String) = {
    val tmp = equCols.map(f => (f, cleanup(newMap(f)).equals(cleanup(oldMap(f)))))
    val eq = tmp.map(_._2).reduce((a1, a2) => a1 && a2)
    if (eq) {
      (true, null)
    } else {
      (eq, tmp.filter(!_._2).map(_._1).mkString(","))
    }
  }
}