package com.winhc.bigdata.spark.ng.jobs

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2020/12/21 10:13
 *        ng空间下企业数据通用处理程序
 */
case class general_handler(s: SparkSession,
                           project: String, //表所在工程名
                           md5_fields: Seq[String],
                           tn: String, is_super_filter: Boolean = true //开启强过滤模式
                          ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  val ods_tab = s"$project.ods_$tn"
  val inc_ods_tab = s"$project.inc_ods_$tn"

  val ads_tab = s"$project.ads_$tn"
  val inc_ads_tab = s"$project.inc_ads_$tn"

  verify()
  reg_udf()

  private def verify(): Unit = {
    val catalog = spark.catalog
    if (!(catalog.tableExists(ods_tab) && catalog.tableExists(inc_ods_tab) && catalog.tableExists(inc_ads_tab) && catalog.tableExists(ads_tab))) {
      println("table is not exists !!!")
      sys.exit(-1)
    }
  }

  private def reg_udf(): Unit = {
    cleanup()
  }

  val rowkey_f = md5_fields.isEmpty match {
    case true => s"company_id"
    case false => s"concat_ws('_',company_id,md5(cleanup(concat_ws('',${md5_fields.mkString(",")}))))"
  }
  val inter_cols = getColumns(ods_tab).intersect(getColumns(inc_ods_tab)).diff(Seq("rowkey"))


  val clean_up =
    s"""
       |company_id <> '0'
       |AND company_id is not null
       |AND trim(company_id) <> ''
       |${
      is_super_filter match {
        //每一个去重字段都不允许为空
        case true => s"AND ${md5_fields.map(" " + _ + " is not null ").mkString("AND")}"
        case false => ""
      }
    }
       |AND trim(concat_ws('',${md5_fields.mkString(",")})) <> ''
       |""".stripMargin

  val up = inter_cols.contains("update_time") match {
    case true => " DESC,update_time"
    case false => ""
  }


  def all(): Unit = {
    val inc_ods_ds = getLastPartitionsOrElse(inc_ods_tab, getLastPartitionsOrElse(ods_tab, null))

    if (inc_ods_ds == null) {
      println("ds is null !!!")
      return
    }

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $ads_tab PARTITION(ds='$inc_ods_ds')
         |SELECT  ${
        getColumns(ads_tab).diff(Seq("ds"))
          .map(f => {
            if (f.equalsIgnoreCase("deleted"))
              "COALESCE(deleted,0) as deleted"
            else
              f
          }).mkString(",")
      }
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds${up} DESC) AS num
         |            FROM    (
         |                        SELECT  $rowkey_f as rowkey
         |                                ,${inter_cols.mkString(",")}
         |                        FROM    $ods_tab
         |                        WHERE   ds > 0
         |                        AND     $clean_up
         |                        UNION ALL
         |                        SELECT  $rowkey_f as rowkey
         |                                ,${inter_cols.mkString(",")}
         |                        FROM    $inc_ods_tab
         |                        WHERE   ds > 0
         |                        AND     $clean_up
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.num = 1
         |""".stripMargin)
  }

  def inc(): Unit = {
    var org_ds = ""
    var target_ds = ""

    val inc_ods_ds = getLastPartitionsOrElse(inc_ods_tab, null)
    val inc_ads_ds = getLastPartitionsOrElse(inc_ads_tab, null)
    val ads_ds = getLastPartitionsOrElse(ads_tab, null)


    if (inc_ods_ds == null) {
      //没有inc_ods_tab数据，直接重跑全量ods数据
      all()
      return
    }
    target_ds = inc_ods_ds

    if (inc_ads_ds == null) {
      org_ds = ads_ds
    } else {
      org_ds = inc_ads_ds
    }

    if (org_ds.equals(target_ds)) {
      val inc_ads_sec_ds = getSecondLastPartitionOrElse(inc_ads_tab, null)
      if (inc_ads_sec_ds == null) {
        org_ds = ads_ds
      } else {
        org_ds = inc_ads_sec_ds
      }
    }


    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_ads_tab PARTITION(ds='$target_ds')
         |SELECT  ${
        getColumns(inc_ads_tab).diff(Seq("ds"))
          .map(f => {
            if (f.equalsIgnoreCase("deleted"))
              "COALESCE(deleted,0) as deleted"
            else
              f
          }).mkString(",")
      }
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds${up} DESC) AS num
         |            FROM    (
         |                        SELECT  $rowkey_f as rowkey
         |                                ,${inter_cols.mkString(",")}
         |                        FROM    $inc_ods_tab
         |                        WHERE   ds > $org_ds
         |                        AND     $clean_up
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.num = 1
         |""".stripMargin)

    addEmptyPartitionOrSkip(inc_ads_tab, target_ds)
  }


  def calc(): Unit = {
    val ads_ds = getLastPartitionsOrElse(ads_tab, null)
    if (ads_ds == null)
      all()
    else
      inc()
  }
}

object general_handler {

  def run(s: SparkSession, project: String, tn: String, md5_fields: Seq[String] = null): Unit = {
    val args_job = args_company_job.get_args_company_job(tn)

    tn match {
      case _ => {
        general_handler(s = s, project = project, tn = tn, md5_fields = args_job.md5_fields, is_super_filter = args_job.is_super_filter).calc()
      }
    }
  }

  def main(args: Array[String]): Unit = {
    val Array(tn) = args
    val project = "winhc_ng"
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName + ":" + tn, config)
    general_handler.run(spark, project, tn, null)
    spark.stop()

  }
}
